//
//  file.cpp
//  Lily
//
//  Created by KMY on 2015/05/06.
//  Copyright (c) 2015年 KMY. All rights reserved.
//

#include "file.h"

File::File() : fileName(0) {
}

File::File(const char* fn) : fileName(0) {
	setFileName(const_cast<char*>(fn));
}

File::~File() {
	delete fileName;
	if (fp) fclose(fp);
}


void File::setFileName(const char *fn) {
	delete fileName;
	Character::strcpyn(fn, &fileName);
}

void File::getFileName(char **output) {
	Character::strcpyn(fileName, output);
}

bool File::open(const char *fn, const char* mode) {
	setFileName(fn);
	return open(mode);
}

bool File::open(const char* mode) {
	fp = fopen(fileName, mode);
	if (!fp) return false;
	return true;
}

void File::close() {
	fclose(fp);
	fp = 0;
}

int File::read(char *output, int maxlen) {
	return static_cast<int>(fread(output, sizeof(char), maxlen, fp));
}

int File::readn(char **output) {
	int fileSize = size();
	*output = new char[fileSize];
	return read(*output, fileSize);
}

int File::write(const char *str, int len) {
	if (!len) len = Character::strlen(str);
	return static_cast<int>(fwrite(str, sizeof(char), len, fp));
}

int File::write(const lchar *str, int len) {
	char* cstr;
	Character::toCharn(str, &cstr);
	int result = write(cstr, len);
	delete cstr;
	return result;
}

bool File::isEnd() {
	return pos()+1 >= size();
}

int File::size() {
	fpos_t tmp, fsize;
	fgetpos(fp, &tmp);
	fseek(fp, 0, SEEK_END);
	fgetpos(fp, &fsize);
	fseek(fp, tmp, SEEK_SET);
	return static_cast<int>(fsize);
}

int File::pos() {
	fpos_t pos;
	fgetpos(fp, &pos);
	return static_cast<int>(pos);
}
