//
//  generate.cpp
//  Lily
//
//  Created by KMY on 2015/05/09.
//  Copyright (c) 2015年 KMY. All rights reserved.
//

#include "generate.h"
#include "script.h"
#include "token.h"
#include "javasource.h"
#include "block.h"
#include "macro.h"
#include "message.h"

Block* Generate::globalBlock = 0;

Generate::Generate(Script* script) {
	scriptObj = script;
}

void Generate::initUp() {
	if (!globalBlock) globalBlock = new Block();
}

void Generate::cleanUp() {
	delete globalBlock;
}

void Generate::start(JavaSource* mainClass) {
	
	// トークン取得
	const lchar* tokenList = scriptObj->getTokenLchar();
	const lchar* tokenType = tokenList;
	const lchar* tokenValue = tokenList+1;
	const lchar* beforeTokenType = 0;
	const lchar* beforeTokenValue = 0;
	
	// Javaソースファイル
	bool mainClassCreateFlag = false;
	if (!mainClass) {
		mainClass = new JavaSource("LilyMain", "jp.kmy.lily");
		mainClass->startJavaMain();
		mainClassCreateFlag = true;
	}
	
	JavaSource* currentClass = mainClass;
	Block* currentBlock = globalBlock;
	
	// マクロスタック
	const lchar* macroStack[_MACROSTACKMAX] = {0};
	int macroStackPointer = 0;
	
	// インデント数のスタック
	int indentStack[_BLOCKNESTMAX] = {0};
	int indentStackPointer = 0;
	int indentTypeStack[_BLOCKNESTMAX] = {0};
	Variable* newIndentVariable = 0;
	int newIndentType = IT_NONAME;
	
	// 関数などの引数
	Variable* firstParam = 0;
	
	// フラグ
	int indent = 0;						// 現在行のインデント数
	bool isStepToken = true;			// トークンを１進めるか
	bool isJava = false;				// Java文章の中であるか
	bool isAnyName = false;
	bool isMacroName = false;			// マクロ名の解読中であるか
	
	// 開始
	while (*tokenType != Token::TOKEN_ENDFILE) {
		
		//Token::tokenTest(tokenType);
		
		// ----------------------------------------------------
		// Javaコード部分（直接出力）
		// ----------------------------------------------------
		
		if (isJava) {
			
			// Java出力の終了
			if (*tokenType == Token::TOKEN_SYMBOL && *tokenValue == Token::SYMBOL_JAVA_END) {
				isJava = false;
			}
			
			// 出力続行
			else {
				
				// 書かれたことをそのまま出力
				if (*tokenType == Token::TOKEN_JAVA_SRC) {
					currentClass->write(tokenValue);
				}
				
				// 埋め込まれた変数を出力
				else if (*tokenType == Token::TOKEN_JAVA_PARAM) {
					
					// 該当する変数を検索
					Variable* obj = currentBlock->searchVariable(tokenValue);
					if (obj) {
						if (!obj->getLiteralFlag()) {
							char* javaName = obj->getJavaName();
							currentClass->write(javaName);
							delete javaName;
						} else {
							currentClass->write(obj->getLiteral());
						}
					}
					else {
						Message::error("変数 %s が見つかりません", tokenValue, Message::MES_CORE_ERROR, scriptObj);
					}
				}
			}
		}
		
		// ----------------------------------------------------
		// 日本語で書かれたプログラムの処理
		// ----------------------------------------------------
		
		else {
			
			// ----------------------------------------------------
			// インデント、ブロック処理
			// ----------------------------------------------------
			
			if (*tokenType == Token::TOKEN_INDENT ||
				(beforeTokenType && *beforeTokenType == Token::TOKEN_YN)) {

				// インデントなしであるか？
				if (*tokenType != Token::TOKEN_INDENT) {
					indent = 0;
				}
				else {
					// インデント更新
					indent = *tokenValue;
				}
				
				// ブロックの開始を確認
				if (indent > indentStack[indentStackPointer]) {
					if (indentStackPointer+1 >= _BLOCKNESTMAX) {
						Message::error("ブロックのネスト数が制限を超えています", Message::MES_CORE_ERROR, scriptObj);
					} else {
						if (newIndentType != IT_MACRO && newIndentType != IT_MACRO_DEF) {
							currentClass->write("{");
						}
						Block* bobj = new Block(currentBlock);
						currentBlock = bobj;
						currentBlock->addVariable(newIndentVariable);
						
						indentStackPointer++;
						indentStack[indentStackPointer] = indent;
						indentTypeStack[indentStackPointer] = newIndentType;
						newIndentType = IT_NONAME;
						newIndentVariable = 0;
					}
				}
				
				// 前行で設定された次のインデントの種類をリセット
				else if (indent == indentStack[indentStackPointer]) {
					newIndentType = IT_NONAME;
				}
				
				else {
					// ブロックの終了を確認
					while (indent < indentStack[indentStackPointer]) {
						
						// ブロックを閉じる
						if (indentTypeStack[indentStackPointer] != IT_MACRO) {
							if (indentTypeStack[indentStackPointer] != IT_MACRO_DEF) {
								currentClass->write("}");
							}
							Block* tmp = currentBlock;
							currentBlock = currentBlock->getParentBlock();
							delete tmp;
						}
						
						// マクロなら元の場所へ戻る
						else {
							macroStackPointer--;
							if (macroStackPointer < 0) {
								Message::error("マクロスタックポインタが0を下回りました", Message::MES_LILY_ERROR, scriptObj);
							} else {
								tokenType = Token::nextToken(macroStack[macroStackPointer]);
								tokenValue = tokenType - 1;
							}
						}
						
						indentStackPointer--;
					}
				}
			}
			
			// ----------------------------------------------------
			// 関数、メソッドの解析（引数を整理し命令を呼び出すコードを出力）
			// 　　マクロは呼び出されるたび定義部分へジャンプするため
			// 　　定義部分は直接処理しない
			// ----------------------------------------------------
			
			if (indentTypeStack[indentStackPointer] != IT_MACRO_DEF) {
			
				// Java文章の開始
				if (*tokenType == Token::TOKEN_SYMBOL && *tokenValue == Token::SYMBOL_JAVA_START) {
					isJava = true;
				}
			
				// マクロ開始を表す記号
				else if (*tokenType == Token::TOKEN_S_MACRO) {
					isAnyName = isMacroName = true;
					newIndentType = IT_MACRO_DEF;
				}
			
				// ----------------------------------------------------
				// 助詞検出時は引数に追加
				// ----------------------------------------------------
				
				else if (beforeTokenType && *tokenType == Token::TOKEN_JOSHI) {
					Variable *obj = new Variable();
					if (*beforeTokenType == Token::TOKEN_UNKNOWN) {
						obj->setName(beforeTokenType+1);
						obj->setType(Variable::VT_UNKNOWN);
					}
					else if (*beforeTokenType == Token::TOKEN_STR) {
						obj->setLiteral(beforeTokenType+1);
						obj->setType(Variable::VT_STR);
					}
					obj->setJoshi(*tokenValue);
					
					if (!firstParam) firstParam = obj;
					else firstParam->add(obj);
				}
				
				// ----------------------------------------------------
				// 改行、「。」など命令終端を見つけた場合
				// ----------------------------------------------------
				
				else if (beforeTokenType &&
						 (*tokenType == Token::TOKEN_ENDINST || *tokenType == Token::TOKEN_YN)) {
					
					
					// ----------------------------------------------------
					// ブロック内における、定義済関数・メソッド・マクロの検出と処理
					// ----------------------------------------------------
					
					if (!isAnyName && *beforeTokenType == Token::TOKEN_UNKNOWN) {
						
						// マクロ：読む場所を移動するだけ
						Macro* mobj = currentBlock->searchMacro(beforeTokenValue, firstParam);
						if (mobj) {
							if (macroStackPointer+1 >= _MACROSTACKMAX) {
								Message::error("マクロのネスト数が制限を超えています", Message::MES_CORE_ERROR, scriptObj);
							} else {
								newIndentType = IT_MACRO;
								newIndentVariable = firstParam;
								macroStack[macroStackPointer] = beforeTokenType;
								macroStackPointer++;
								firstParam = 0;
								
								// 仮引数の名前をコピーする
								newIndentVariable->setEveryName(mobj->getFirstParam());
							
								// 読むトークンを移動
								tokenType = mobj->getStartToken();
								tokenValue = tokenType + 1;
								isStepToken = false;
							}
						}
						
						else {
							Message::error("不明なトークン %s が見つかりました", beforeTokenValue, Message::MES_CORE_ERROR, scriptObj);
						}
					}
					
					isAnyName = isMacroName = false;
				}
			}
		}
		
		// ---------------------------------
		
		// 次のトークンへ
		if (isStepToken) {
			beforeTokenType = tokenType;
			beforeTokenValue = tokenValue;
			tokenType = Token::nextToken(tokenType);
			tokenValue = tokenType + 1;
		}
		isStepToken = true;
	}
	
	if (mainClassCreateFlag) {
		mainClass->endJavaMain();
		delete mainClass;
	}
	
}

void Generate::addGlobalMacro(Macro *obj) {
	globalBlock->addMacro(obj);
}
