//
//  script.cpp
//  Lily
//
//  Created by KMY on 2015/05/08.
//  Copyright (c) 2015年 KMY. All rights reserved.
//

#include "script.h"
#include "token.h"
#include "generate.h"
#include "javasource.h"
#include "message.h"
#include "preprocessor.h"

Script::Script() : File() {
	init();
}

Script::Script(const char* fn) : File(fn) {
	init();
}

void Script::init() {
	scriptIntoChar = 0;
	scriptIntoLchar = 0;
	tokenObj = 0;
	preProcessorObj = 0;
	generateObj = 0;
	linenum = 1;
}

Script::~Script() {
	delete scriptIntoChar;
	delete scriptIntoLchar;
	delete tokenObj;
	delete preProcessorObj;
	delete generateObj;
}

const char* Script::getScriptIntoChar() {
	return scriptIntoChar;
}

const lchar* Script::getScriptIntoLchar() {
	return scriptIntoLchar;
}

const lchar* Script::getTokenLchar() {
	return tokenObj->getTokenLchar();
}

int Script::getLineNumber() {
	return linenum;
}

void Script::setLineNumber(int num) {
	linenum = num;
}

int Script::nextLine() {
	return ++linenum;
}

bool Script::load(JavaSource* mainClass) {
	
	// ２回以上の実行なら終わる
	if (scriptIntoChar) {
		Message::error("同一ファイルオブジェクトを複数回参照しようとしました", Message::MES_LILY_ERROR);
		return false;
	}
	
	// ファイルを読む
	if (!File::open("rb")) {
		return false;
	}
	File::readn(&scriptIntoChar);
	Character::toLcharn(scriptIntoChar, &scriptIntoLchar);
	
	// 処理
	token();
	preProcessor();
	generate(mainClass);
	compile();
	
	return true;
}

void Script::token() {
	tokenObj = new Token(this);
	tokenObj->analysis();
}

void Script::preProcessor() {
	preProcessorObj = new PreProcessor(this);
	preProcessorObj->start();
}

void Script::generate(JavaSource* mainClass) {
	generateObj = new Generate(this);
	generateObj->start(mainClass);
}

void Script::compile() {
	
}

