//
//  variable.cpp
//  Lily
//
//  Created by KMY on 2015/05/09.
//  Copyright (c) 2015年 KMY. All rights reserved.
//

#include "variable.h"
#include "message.h"

int Variable::count = 0;

Variable::Variable() {
	init();
}

Variable::Variable(const lchar* vn, int vt, bool literalFlagp) {
	init();
	if (!literalFlag)
		setName(vn);
	else {
		literalFlag = true;
		literal = vn;
	}
	setType(vt);
}

Variable::Variable(const lchar* vn, const lchar* vtn) {
	init();
	setName(vn);
	type = VT_USER;
	Character::lstrcpyn(vtn, &typeName);
}

void Variable::init() {
	nextObj = 0;
	lastObj = 0;
	name = 0;
	type = 0;
	typeName = 0;
	arrayFlag = false;
	literalFlag = false;
	value = count++;
}

Variable::~Variable() {
	delete name;
	delete nextObj;
	delete typeName;
}

Variable* Variable::clone() {
	int counttmp = count;
	count = value;
	
	Variable* obj = new Variable();
	obj->setName(name);
	obj->setType(type);
	if (literal)
		obj->setLiteral(literal);
	obj->setJoshi(joshi);
	obj->setArrayFlag(arrayFlag);
	if (obj->next()) {
		nextObj = obj->next()->clone();
	}
	
	count = counttmp;
	return obj;
}

void Variable::setArrayFlag(bool v) {
	arrayFlag = v;
}
bool Variable::getArrayFlag() {
	return arrayFlag;
}

bool Variable::getLiteralFlag() {
	return literalFlag;
}

Variable* Variable::next() {
	return nextObj;
}

Variable* Variable::getLast() {
	return lastObj;
}

Variable* Variable::searchLast() {
	Variable *obj = nextObj;
	while (obj)
		obj = obj->next();
	lastObj = obj;
	return obj;
}

void Variable::setJoshi(int j) {
	joshi = j;
}

int Variable::getJoshi() {
	return joshi;
}

const lchar* Variable::getLiteral() {
	return literal;
}

void Variable::setLiteral(const lchar *l) {
	literal = l;
	literalFlag = true;
}

char* Variable::getJavaName() {
	char* obj = new char[16];
	sprintf(obj, "c%d", value);
	return obj;
}

char* Variable::getJavaType() {
	// スタブ（まだ書いてない）
	return 0;
}

const lchar* Variable::getName() {
	return name;
}

void Variable::setName(const lchar *vn) {
	delete name;
	Character::lstrcpyn(vn, &name);
}

void Variable::setEveryName(Variable *obj) {
	Variable* n = obj;
	while (n) {
		setName(obj->getName());
		n = obj->next();
	}
}

int Variable::getType() {
	return type;
}

int Variable::getLilyType(const lchar *typeName) {
	if (Character::equal(typeName, "数値")) {
		return VT_INT;
	} else if (Character::equal(typeName, "文字列")) {
		return VT_STR;
	} else {
		return VT_UNKNOWN;
	}
}

void Variable::setType(int vt) {
	type = vt;
}

int Variable::getValue() {
	return value;
}

Variable* Variable::add(const lchar *vn, int vt) {
	return add(new Variable(vn, vt));
}

Variable* Variable::add(const lchar *vn, const lchar* vtn) {
	return add(new Variable(vn, vtn));
}

Variable* Variable::add(Variable* obj) {
	if (!nextObj) {
		nextObj = obj;
		lastObj = obj->searchLast();
	}
	else if (!lastObj) {
		lastObj = obj->searchLast();
		if (lastObj) lastObj->nextObj = obj;
	}
	return obj;
}

bool Variable::checkVariable(int vt, const lchar *vn) {
	return Character::equal(vn, name) && type == vt;
}

bool Variable::checkVariable(Variable *obj, bool checkNameFlag) {
	return obj->type == type &&
			obj->arrayFlag == arrayFlag &&
			(!checkNameFlag || literalFlag || Character::equal(obj->name, name)) &&
			obj->getJoshi() == joshi;
}

void Variable::variableTest() {
	char* nn;
	Character::toCharn(literalFlag ? literal : name, &nn);
	printf(
		   literalFlag ? "\"%s\"（%s:リテラル）\t" : "%s（%s）\t",
		   nn,
		   (type == VT_INT ? "整数32" :
			type == VT_STR ? "文字列W" :
			type == VT_VOID ? "型なし" :
			type == VT_USER ? "ユーザ定義" :
			type == VT_UNKNOWN ? "不明" : "未定義"
			));
	delete nn;
}
